from typing import Any, Dict, List, Optional

from .base import Base

class CryptoWallets(Base):
    @property
    def crypto_wallet_url(self):
        return self.get_url("crypto_wallets")

    def authenticate_start(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        user_id: Optional[str] = None,
    ):
        data = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
        }
        if user_id:
            data["user_id"] = user_id

        return self._post(
            "{0}/authenticate/start".format(
                self.crypto_wallet_url,
            ),
            data=data,
        )

    def authenticate(
        self,
        crypto_wallet_address: str,
        crypto_wallet_type: str,
        signature: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "crypto_wallet_address": crypto_wallet_address,
            "crypto_wallet_type": crypto_wallet_type,
            "signature": signature,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims

        return self._post(
            "{0}/authenticate".format(
                self.crypto_wallet_url,
            ),
            data=data,
        )



from typing import Any, Dict, Optional

from .base import _validate_attributes, Base

class MagicLinks(Base):
    def __init__(self, client):
        super().__init__(client)
        self.email = Email(client)

    @property
    def magic_link_url(self):
        return self.get_url("magic_links")

    def create(
        self,
        user_id: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
            "user_id": user_id,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        if attributes:
            data["attributes"] = attributes
        return self._post(
            self.magic_link_url,
            data=data,
        )

    def authenticate(
        self,
        token: str,
        attributes: Optional[Dict] = None,
        options: Optional[Dict] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        code_verifier: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        if options:
            options = self._validate_options(options)

        data: Dict[str, Any] = {
            "token": token,
        }
        if attributes:
            data["attributes"] = attributes
        if options:
            data["options"] = options

        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims
        if code_verifier:
            data["code_verifier"] = code_verifier

        return self._post(
            "{0}/authenticate".format(self.magic_link_url),
            data=data,
        )

class Email(Base):
    @property
    def magic_link_url(self):
        return self.get_url("magic_links")

    def send(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        code_challenge: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
            "email": email,
            "attributes": attributes,
        }
        if login_magic_link_url:
            data["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url:
            data["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes:
            data["signup_expiration_minutes"] = signup_expiration_minutes
        if code_challenge:
            data["code_challenge"] = code_challenge

        return self._post(
            "{0}/email/send".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def login_or_create(
        self,
        email: str,
        login_magic_link_url: Optional[str] = None,
        signup_magic_link_url: Optional[str] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        create_user_as_pending: Optional[bool] = False,
        code_challenge: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
           "email": email,
           "attributes": attributes,
           "create_user_as_pending": create_user_as_pending,
        }
        if login_magic_link_url:
            data["login_magic_link_url"] = login_magic_link_url
        if signup_magic_link_url:
            data["signup_magic_link_url"] = signup_magic_link_url
        if login_expiration_minutes:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes:
            data["signup_expiration_minutes"] = signup_expiration_minutes
        if code_challenge:
            data["code_challenge"] = code_challenge

        return self._post(
            "{0}/email/login_or_create".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def invite(
        self,
        email: str,
        invite_magic_link_url: Optional[str] = None,
        invite_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        middle_name: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
            "email": email,
            "attributes": attributes,
            "name": {
                "first_name": first_name,
                "middle_name": middle_name,
                "last_name": last_name,
            },
        }
        if invite_magic_link_url:
            data["invite_magic_link_url"] = invite_magic_link_url
        if invite_expiration_minutes:
            data["invite_expiration_minutes"] = invite_expiration_minutes

        return self._post(
            "{0}/email/invite".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def revoke_invite(
        self,
        email: str,
    ):
        return self._post(
            "{0}/email/revoke_invite".format(
                self.magic_link_url,
            ),
            data={
                "email": email,
            },
        )



class StytchError(Exception):
    def __init__(
        self,
        status_code: int = None,
        request_id: str = None,
        error_type: str = None,
        error_message: str = None,
        error_url: str = None,
        **kwargs
    ):
        self.status_code = status_code
        self.request_id = request_id
        self.error_type = error_type
        self.error_message = error_message
        self.error_url = error_url

    def __repr__(self):
        return "StytchError {0}".format(self.__dict__)

    def __str__(self):
        return str(self.__dict__)

class ClientError(Exception):
    def __init__(self, code: str, message: str, cause: Exception = None):
        super().__init__(self, message)
        self.code = code
        self._message = message
        self.cause = cause

    def __repr__(self):
        return "{}({!r}, {!r}, {!r})".format(self.__class__.__name__, self.code, self._message, self.cause)

    def __str__(self):
        s = "({}) {}".format(self.code, self._message)
        if self.cause:
            return "{}: {}".format(s, self.cause)
        return s



from typing import Any, Dict, List, Optional

from .base import Base

class WebAuthn(Base):
    @property
    def webauthn_url(self):
        return self.get_url("webauthn")

    def register_start(
        self,
        user_id: str,
        domain: str,
        user_agent: Optional[str] = None,
        authenticator_type: Optional[str] = None,
    ):
        data = {
            "user_id": user_id,
            "domain": domain,
        }
        if user_agent:
            data["user_agent"] = user_agent
        if authenticator_type:
            data["authenticator_type"] = authenticator_type

        return self._post(
            "{0}/register/start".format(
                self.webauthn_url,
            ),
            data=data,
        )

    def register(
        self,
        user_id: str,
        public_key_credential: str,
    ):
        data = {
            "user_id": user_id,
            "public_key_credential": public_key_credential,
        }

        return self._post(
            "{0}/register".format(
                self.webauthn_url,
            ),
            data=data,
        )

    def authenticate_start(
        self,
        user_id: str,
        domain: str,
    ):
        data = {
            "user_id": user_id,
            "domain": domain,
        }

        return self._post(
            "{0}/authenticate/start".format(
                self.webauthn_url,
            ),
            data=data,
        )

    def authenticate(
        self,
        public_key_credential: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "public_key_credential": public_key_credential,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims

        return self._post(
            "{0}/authenticate".format(
                self.webauthn_url,
            ),
            data=data,
        )



from typing import Any, Dict, List, Optional, cast
from typing_extensions import TypedDict
import time

import jwt

from .base import _validate_attributes, Base
from .error import ClientError


class JWTInvalidError(ClientError):
    def __init__(self, message: str, cause: Exception = None):
        super().__init__("jwt_invalid", message, cause)


class JWTTooOldError(ClientError):
    def __init__(self, message: str, cause: Exception = None):
        super().__init__("jwt_too_old", message, cause)


_session_claim = "https://stytch.com/session"

SessionClaim = TypedDict(
    "SessionClaim",
    {
        "id": str,
        "started_at": str,
        "last_accessed_at": str,
        "expires_at": Optional[str],
        "attributes": Dict[str, str],
        "authentication_factors": List[Dict[str, Any]],
    },
)


class Sessions(Base):
    def __init__(self, client, jwks_client):
        super().__init__(client)

        self._jwks_client = jwks_client
        self._jwt_audience = client.project_id
        self._jwt_issuer = "stytch.com/" + client.project_id

    @property
    def sessions_url(self):
        return self.get_url("sessions")

    def get(
        self,
        user_id: str,
    ):
        query_params = {
            "user_id": user_id,
        }
        return self._get(self.sessions_url, query_params)

    def authenticate(
        self,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {}

        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims

        return self._post(
            "{0}/authenticate".format(self.sessions_url),
            data=data,
        )

    def authenticate_jwt(
        self,
        session_jwt: str,
        *,
        max_token_age_seconds: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        """Parse a JWT and verify the signature, preferring local verification over remote.

        If max_token_age_seconds is set, remote verification will be forced if the JWT was issued
        at (based on the "iat" claim) more than that many seconds ago.

        To force remote validation for all tokens, set max_token_age_seconds to zero or use the
        authenticate method instead.
        """
        try:
            session = self.authenticate_jwt_local(
                session_jwt,
                max_token_age_seconds=max_token_age_seconds,
            )
            return {
                "session": session,
                "session_jwt": session_jwt,
            }
        except Exception as e:
            # JWT could not be verified locally. Check with the Stytch API.
            return self.authenticate(session_custom_claims=session_custom_claims, session_jwt=session_jwt).json()

    def authenticate_jwt_local(
        self,
        session_jwt: str,
        *,
        max_token_age_seconds: Optional[int] = None,
        leeway: int = 0,
    ):
        """Parse a JWT and verify the signature locally (without calling /authenticate in the API).

        If max_token_age_seconds is set, this will return an error if the JWT was issued (based on
        the "iat" claim) more than maxTokenAge seconds ago.

        If max_token_age_seconds is explicitly set to zero, all tokens will be considered too old,
        even if they are otherwise valid.

        The value for leeway is the maximum allowable difference in seconds when comparing
        timestamps. It defaults to zero.
        """
        now = time.time()

        signing_key = self._jwks_client.get_signing_key_from_jwt(session_jwt)

        payload = jwt.decode(
            session_jwt,
            signing_key.key,
            algorithms=["RS256"],
            options={
                "require": ["aud", "iss", "exp", "iat", "nbf"],
                "verify_signature": True,
                "verify_aud": True,
                "verify_iss": True,
                "verify_exp": True,
                "verify_iat": True,
                "verify_nbf": True,
            },
            audience=self._jwt_audience,
            issuer=self._jwt_issuer,
            leeway=leeway,
            # The max_token_age_seconds value is applied after decoding.
        )

        if max_token_age_seconds is not None:
            iat = payload["iat"]
            if now - iat >= max_token_age_seconds:
                message = "JWT was issued at {}, more than {} seconds ago".format(
                    iat, max_token_age_seconds
                )
                raise JWTTooOldError(message)

        # Unpack the session claim to match the detached session format.
        claim = cast(SessionClaim, payload[_session_claim])

        # For JWTs that include it, prefer the inner expires_at claim.
        expires_at = claim["expires_at"] or time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(payload["exp"]))

        return {
            "user_id": payload["sub"],
            "expires_at": expires_at,
            "session_id": claim["id"],
            "attributes": claim["attributes"],
            "authentication_factors": claim["authentication_factors"],
            "started_at": claim["started_at"],
            "last_accessed_at": claim["last_accessed_at"],
        }

    def revoke(
        self,
        session_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ):
        data = {}
        if session_id:
            data["session_id"] = session_id
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt

        return self._post(
            "{0}/revoke".format(self.sessions_url),
            data=data,
        )

    def jwks(self, project_id: str):
        return self._get("{0}/jwks/{1}".format(self.sessions_url, project_id))



import warnings

import jwt

from .api.users import Users
from .api.magic_links import MagicLinks
from .api.otp import OTP
from .api.oauth import OAuth
from .api.sessions import Sessions
from .api.totps import TOTPs
from .api.webauthn import WebAuthn
from .api.crypto_wallets import CryptoWallets
from .api.passwords import Passwords

class Client:
    """
    Stytch API Python client.

    Learn more at https://stytch.com/docs
    """

    def __init__(
        self,
        project_id: str,
        secret: str,
        environment: str,
        suppress_warnings: bool = False,
    ):
        self.project_id = project_id
        self.secret = secret
        self.environment = environment

        self.base_url = _env_url(environment, suppress_warnings)

        jwks_url = self.base_url + 'sessions/jwks/' + self.project_id
        jwks_client = jwt.PyJWKClient(jwks_url)

        self.users = Users(self)
        self.magic_links = MagicLinks(self)
        self.oauth = OAuth(self)
        self.otps = OTP(self)
        self.sessions = Sessions(self, jwks_client)
        self.webauthn = WebAuthn(self)
        self.totps = TOTPs(self)
        self.crypto_wallets = CryptoWallets(self)
        self.passwords = Passwords(self)

def _env_url(env: str, suppress_warnings: bool = False) -> str:
    '''Resolve the base URL for the Stytch API environment.
    '''

    # Supported production environments
    if env == "test":
        if not suppress_warnings:
            warnings.warn("Test version of Stytch not intended for production use")
        return "https://test.stytch.com/v1/"
    elif env == "live":
        return "https://api.stytch.com/v1/"

    # Internal development override. URL builders assume the base URL has a
    # trailing slash, so add one if it's missing.
    if not env.endswith('/'):
        return env + '/'
    return env



from enum import Enum
from typing import Any, Dict, Optional

from .base import Base, _validate_attributes


class OAuth(Base):
    def __init__(self, client):
        super().__init__(client)

    @property
    def oauth_url(self):
        return self.get_url("oauth")

    def authenticate(
        self,
        token: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        code_verifier: Optional[str] = None,
    ):

        data: Dict[str, Any] = {
            "token": token,
        }

        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims
        if code_verifier:
            data["code_verifier"] = code_verifier

        return self._post(
            "{0}/authenticate".format(self.oauth_url),
            data=data,
        )



import pytest
from stytch.client import Client


class TestStytchClient:
    def test_test_env_returns_base_url(self):
        client = Client("project_id", "secret", "test", suppress_warnings=True)
        assert client.base_url == "https://test.stytch.com/v1/"

    def test_production_env_returns_prod_url(self):
        client = Client("project_id", "secret", "live", suppress_warnings=True)
        assert client.base_url == "https://api.stytch.com/v1/"

    def test_custom_env_for_development(self):
        client = Client("project_id", "secret", "https://localhost:8000/v1", suppress_warnings=True)
        assert client.base_url == "https://localhost:8000/v1/"

        client = Client("project_id", "secret", "https://localhost:8000/v1/", suppress_warnings=True)
        assert client.base_url == "https://localhost:8000/v1/"

    def test_users_controller_exists(self):
        client = Client("project_id", "secret", "test", suppress_warnings=True)
        assert client.users
        assert client.users.get
        assert client.users.update
        assert client.users.delete
        assert client.users.create

    def test_magic_links_controller_exists(self):
        client = Client("project_id", "secret", "test", suppress_warnings=True)
        assert client.magic_links
        assert client.magic_links.email.send
        assert client.magic_links.authenticate



from typing import Any, Dict, List, Optional

from .base import Base, _validate_attributes

class Passwords(Base):
    def __init__(self, client):
        super().__init__(client)
        self.email = Email(client)

    @property
    def password_url(self):
        return self.get_url("passwords")

    def create(
        self,
        email: str,
        password: str,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "email": email,
            "password": password,
        }
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims

        return self._post(self.password_url, data=data)

    def authenticate(
        self,
        email: str,
        password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "email": email,
            "password": password,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims

        return self._post(
            "{0}/authenticate".format(
                self.password_url,
            ),
            data=data,
        )

    def strength_check(
        self,
        password: str,
        email: Optional[str] = None,
    ):
        data: Dict[str, Any] = {
            "password": password,
        }
        if email:
            data["email"] = email

        return self._post(
            "{0}/strength_check".format(
                self.password_url,
            ),
            data=data,
        )

    def migrate(
        self,
        email: str,
        hash: str,
        hash_type: str,
        md_5_config: Optional[Dict[str, Any]] = None,
        argon_2_config: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "email": email,
            "hash": hash,
            "hash_type": hash_type,
        }
        if md_5_config:
            data["md_5_config"] = md_5_config
        if argon_2_config:
            data["argon_2_config"] = argon_2_config

        return self._post(
            "{0}/migrate".format(
                self.password_url,
            ),
            data=data,
        )

class Email(Base):
    @property
    def password_url(self):
        return self.get_url("passwords/email")

    def reset_start(
        self,
        email: str,
        login_redirect_url: Optional[str] = None,
        reset_password_redirect_url: Optional[str] = None,
        reset_password_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        code_challenge: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
            "email": email,
        }
        if login_redirect_url:
            data["login_redirect_url"] = login_redirect_url
        if reset_password_redirect_url:
            data["reset_password_redirect_url"] = reset_password_redirect_url
        if reset_password_expiration_minutes:
            data["reset_password_expiration_minutes"] = reset_password_expiration_minutes
        if attributes:
            data["attributes"] = attributes
        if code_challenge:
            data["code_challenge"] = code_challenge

        return self._post(
            "{0}/reset/start".format(
                self.password_url,
            ),
            data=data,
        )

    def reset(
        self,
        token: str,
        password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        attributes: Optional[Dict] = None,
        options: Optional[Dict] = None,
        code_verifier: Optional[str] = None,
    ):
        if attributes:
            attributes = _validate_attributes(attributes)

        data: Dict[str, Any] = {
            "token": token,
            "password": password,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims
        if attributes:
            data["attributes"] = attributes
        if options:
            data["options"] = options
        if code_verifier:
            data["code_verifier"] = code_verifier

        return self._post(
            "{0}/reset".format(
                self.password_url,
            ),
            data=data,
        )



import json
import requests

from .error import StytchError


def throw_stytch_exception(func):
    def wrapper(*args, **kwargs):
        resp: requests.models.Response = func(*args, **kwargs)
        if resp.status_code >= 400:
             raise StytchError(**json.loads(resp._content))
        else:
            return resp

    return wrapper



from typing import Any, Dict, Optional

from .base import Base


class TOTPs(Base):
    def __init__(self, client):
        super().__init__(client)

    @property
    def totps_url(self):
        return self.get_url("totps")

    def create(
        self,
        user_id: str,
        expiration_minutes: Optional[int] = None,
    ):
        data: Dict[str, Any] = {
            "user_id": user_id,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        return self._post(
            self.totps_url,
            data=data,
        )

    def authenticate(
        self,
        user_id: str,
        totp_code: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "user_id": user_id,
            "totp_code": totp_code,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims
        return self._post(
            "{0}/authenticate".format(self.totps_url),
            data=data,
        )

    def recovery_codes(
        self,
        user_id: str,
    ):
        data={
            "user_id": user_id,
        }
        return self._post(
            "{0}/recovery_codes".format(self.totps_url),
            data=data,
        )

    def recover(
        self,
        user_id: str,
        recovery_code: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
    ):
        data: Dict[str, Any] = {
            "user_id": user_id,
            "recovery_code": recovery_code,
        }
        if session_token:
            data["session_token"] = session_token
        if session_jwt:
            data["session_jwt"] = session_jwt
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims:
            data["session_custom_claims"] = session_custom_claims
        return self._post(
            "{0}/recover".format(self.totps_url),
            data=data,
        )
